@@ -35,14 +35,12 @@
 #' @param it A character vector of length 1 giving the image type to be used. Currently only
 #' "tiff" and "jpeg" are admitted. Default is "jpeg".
 #' @param parallel Numeric. Controls whether parallel computing is applied.
-#'  It specifies the number of cores to be used. Default is 1 (i.e. no parallel computing). 
-#'  Not available in Windows OS.
+#'  It specifies the number of cores to be used. Default is 1 (i.e. no parallel computing).
 #' @param path Character string containing the directory path where the sound files are located. 
 #' If \code{NULL} (default) then the current working directory is used. 
 #' @param sp Data frame with acoustic parameters as the one generated by \code{\link{specan}}. Must contain 'sound.files'
 #' and "selec' columns and the same selections as in 'X'.
-#' @param pb Logical argument to control progress bar. Default is \code{TRUE}. Note that progress bar is only used
-#' when parallel = 1.
+#' @param pb Logical argument to control progress bar. Default is \code{TRUE}.
 #' @param grid Logical argument to control the presence of a grid on the spectrograms (default is \code{TRUE}).
 #' @param clip.edges Logical argument to control whether edges (start or end of signal) in
 #' which amplitude values above the threshold were not detected will be removed when using dfDTW and 
@@ -149,7 +147,7 @@
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu}). It uses 
 #' internally a modified version of the \code{\link[seewave]{spectro}} function from 
 #' seewave package to create spectrograms.
-#last modification on feb-17-2017 (MAS)
+#last modification on mar-13-2018 (MAS)
 
 compare.methods <- function(X = NULL, flim = c(0, 22), bp = c(0, 22), mar = 0.1, wl = 512, ovlp = 90, 
     res = 150, n = 10, length.out = 30, methods = c("XCORR","dfDTW", "ffDTW", "SP"),
@@ -180,13 +178,6 @@ compare.methods <- function(X = NULL, flim = c(0, 22), bp = c(0, 22), mar = 0.1,
   if(!is.numeric(parallel)) stop("'parallel' must be a numeric vector of length 1") 
   if(any(!(parallel %% 1 == 0),parallel < 1)) stop("'parallel' should be a positive integer")
   
-  #if parallel and pb in windows
-  if(parallel > 1 &  pb & Sys.info()[1] == "Windows") {
-    message("parallel with progress bar is currently not available for windows OS")
-    message("running parallel without progress bar")
-    pb <- FALSE
-  } 
-  
   # methods 
   if(any(!is.character(methods),length(methods) > 2)) stop("'methods' must be a character vector of length 2")
   if(length(methods[!methods %in% c("XCORR", "dfDTW", "ffDTW", "SP")]) > 0) 
@@ -232,7 +223,7 @@ compare.methods <- function(X = NULL, flim = c(0, 22), bp = c(0, 22), mar = 0.1,
   #return warning if not all sound files were found
   fs <- list.files(path = getwd(), pattern = "\\.wav$", ignore.case = TRUE)
   if(length(unique(X$sound.files[(X$sound.files %in% fs)])) != length(unique(X$sound.files))) 
-    message(paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% fs)])), 
+    cat(paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% fs)])), 
                   ".wav file(s) not found"))
   
   #count number of sound files in working directory and if 0 stop
@@ -328,9 +319,9 @@ compare.methods <- function(X = NULL, flim = c(0, 22), bp = c(0, 22), mar = 0.1,
   
   if(nrow(X) == 4)  {n <- 1
   combs <- as.matrix(1:4)
-  message("Only 1 possible combination of signals")
+  cat("Only 1 possible combination of signals")
   } else if(n > ncol(combs)) {n <- ncol(combs)
-  message(paste("Only",n, "possible combinations of signals"))
+  cat(paste("Only",n, "possible combinations of signals"))
   }
   
   if(nrow(X) > 4)  combs <- as.data.frame(combs[,sample(1:ncol(combs), n)])
@@ -355,7 +346,7 @@ compare.methods <- function(X = NULL, flim = c(0, 22), bp = c(0, 22), mar = 0.1,
   # screen 10:11 method labels
   
   
-  if(any(parallel == 1, Sys.info()[1] == "Linux") & pb)  message("saving graphs as image files:")
+  if(pb)  cat("saving graphs as image files:")
   
       comp.methFUN <- function(X, u, res, disim.mats, m, mar, flim)
     {
@@ -500,68 +491,16 @@ compare.methods <- function(X = NULL, flim = c(0, 22), bp = c(0, 22), mar = 0.1,
   invisible(dev.off())
   on.exit(invisible(close.screen(all.screens = TRUE)))
   }
-
       
-      options(warn = -1)
+      # set pb options 
+      pbapply::pboptions(type = ifelse(pb, "timer", "none"))
       
-      #parallel not available on windows
-      if(parallel > 1 & Sys.info()[1] == "Windows")
-      {message("parallel computing not availabe in Windows OS for this function")
-        parallel <- 1}
+      # set clusters for windows OS
+      if (Sys.info()[1] == "Windows" & parallel > 1)
+        cl <- parallel::makePSOCKcluster(getOption("cl.cores", parallel)) else cl <- parallel
       
-      if(parallel > 1) {
-        if(Sys.info()[1] == "Windows") {
-           
-        u <- NULL #only to avoid non-declared objects
-         
-         cl <- parallel::makeCluster(parallel)
-         
-         doParallel::registerDoParallel(cl)
-         
-         a1 <- foreach::foreach(u = 1:ncol(combs)) %dopar% {
-           comp.methFUN(X, u, res, disim.mats, m, mar, flim)
-         }
-         
-         parallel::stopCluster(cl)
-         
-      } 
-        
-        if(Sys.info()[1] == "Linux"){    # Run parallel in other operating systems
-        
-          if(pb)
-        a1 <- pbmcapply::pbmclapply(1:ncol(combs), mc.cores = parallel, function(u) {
-          comp.methFUN(X, u, res, disim.mats, m, mar, flim) 
-        }) else
-          
-        a1 <- parallel::mclapply(1:ncol(combs), mc.cores = parallel, function(u) {
-          comp.methFUN(X, u, res, disim.mats, m, mar, flim)
-        })
-        
-      }
-        if(!any(Sys.info()[1] == c("Linux", "Windows"))) # parallel in OSX
-        {
-          cl <- parallel::makeForkCluster(getOption("cl.cores", parallel))
-          
-          doParallel::registerDoParallel(cl)
-          
-          a1 <- foreach::foreach(u = 1:ncol(combs)) %dopar% {
-            comp.methFUN(X, u, res, disim.mats, m, mar, flim)
-          }
-          parallel::stopCluster(cl)
-        }
-        
-        
-        } else {
-          if(pb)
-            a1 <- pbapply::pblapply(1:ncol(combs), function(u) 
-      { 
-        comp.methFUN(X, u, res, disim.mats, m, mar, flim)
-      }) else       
-        a1 <- lapply(1:ncol(combs), function(u) 
+      a1 <- pbapply::pblapply(X = 1:ncol(combs), cl = cl, FUN = function(u) 
       { 
         comp.methFUN(X, u, res, disim.mats, m, mar, flim)
       })
-      
-      }
-
       }
