@@ -2,7 +2,7 @@
 #' 
 #' \code{dfts} extracts the dominant frequency values as a time series.
 #' of signals selected by \code{\link{manualoc}} or \code{\link{autodetec}}.
-#' @usage dfts(X, wl = 512, wl.freq = 512, length.out = 20, wn = "hanning", ovlp = 70, 
+#' @usage dfts(X, wl = 512, wl.freq = 512, length.out = 20, wn = "hanning", ovlp = 70,
 #' bp = c(0, 22), threshold = 15, threshold.time = NULL, threshold.freq = NULL, 
 #' img = TRUE, parallel = 1, path = NULL, img.suffix = "dfts", pb = TRUE, 
 #' clip.edges = FALSE, leglab = "dfts", frange.detec = FALSE, fsmooth = 0.1, ...)
@@ -31,12 +31,10 @@
 #'  'threshold' value is used.
 #' @param parallel Numeric. Controls whether parallel computing is applied.
 #'  It specifies the number of cores to be used. Default is 1 (i.e. no parallel computing).
-#'  Not available in Windows OS.
 #' @param path Character string containing the directory path where the sound files are located.
 #' @param img.suffix A character vector of length 1 with a sufix (label) to add at the end of the names of 
 #' image files.
-#' @param pb Logical argument to control progress bar. Default is \code{TRUE}. Note that progress bar is only used
-#' when parallel = 1.
+#' @param pb Logical argument to control progress bar. Default is \code{TRUE}.
 #' @param clip.edges Logical argument to control whether edges (start or end of signal) in
 #' which amplitude values above the threshold were not detected will be removed. If 
 #' \code{TRUE} this edges will be excluded and signal contour will be calculated on the
@@ -81,13 +79,13 @@
 #' 
 #' }
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu})
-#last modification on oct-26-2016 (MAS)
+#last modification on march-12-2018 (MAS)
 
 dfts <-  function(X, wl = 512, wl.freq = 512, length.out = 20, wn = "hanning", ovlp = 70, 
                   bp = c(0, 22), threshold = 15, threshold.time = NULL, threshold.freq = NULL,
                   img = TRUE, parallel = 1,
                   path = NULL, img.suffix = "dfts", pb = TRUE, clip.edges = FALSE, leglab = "dfts", frange.detec = FALSE, fsmooth = 0.1, ...){     
-
+  
   # reset working directory 
   wd <- getwd()
   on.exit(setwd(wd))
@@ -97,17 +95,15 @@ dfts <-  function(X, wl = 512, wl.freq = 512, length.out = 20, wn = "hanning", o
     setwd(path)
   }  
   
-    #if X is not a data frame
-    if(!class(X) %in% c("data.frame", "selection.table")) stop("X is not of a class 'data.frame' or 'selection table")
-    
-    
+  #if X is not a data frame
+  if(!class(X) %in% c("data.frame", "selection.table")) stop("X is not of a class 'data.frame' or 'selection table")
   
   if(!all(c("sound.files", "selec", 
             "start", "end") %in% colnames(X))) 
     stop(paste(paste(c("sound.files", "selec", "start", "end")[!(c("sound.files", "selec", 
                                                                    "start", "end") %in% colnames(X))], collapse=", "), "column(s) not found in data frame"))
   
-
+  
   #if there are NAs in start or end stop
   if(any(is.na(c(X$end, X$start)))) stop("NAs found in start and/or end")  
   
@@ -136,7 +132,7 @@ dfts <-  function(X, wl = 512, wl.freq = 512, length.out = 20, wn = "hanning", o
   #return warning if not all sound files were found
   recs.wd <- list.files(pattern = "\\.wav$", ignore.case = TRUE)
   if(length(unique(X$sound.files[(X$sound.files %in% recs.wd)])) != length(unique(X$sound.files)) & pb) 
-    message(paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% recs.wd)])), 
+    cat(paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% recs.wd)])), 
                   ".wav file(s) not found"))
   
   #count number of sound files in working directory and if 0 stop
@@ -144,95 +140,82 @@ dfts <-  function(X, wl = 512, wl.freq = 512, length.out = 20, wn = "hanning", o
   if(length(d) == 0){
     stop("The .wav files are not in the working directory")
   }  else 
-  X <- X[d, ]
+    X <- X[d, ]
   
   #if parallel is not numeric
   if(!is.numeric(parallel)) stop("'parallel' must be a numeric vector of length 1") 
   if(any(!(parallel %% 1 == 0),parallel < 1)) stop("'parallel' should be a positive integer")
   
-  #if parallel and pb in windows
-  if(parallel > 1 &  pb & Sys.info()[1] == "Windows") {
-    message("parallel with progress bar is currently not available for windows OS")
-    message("running parallel without progress bar")
-    pb <- FALSE
-  } 
-  
-  #if parallel
-  if(all(parallel > 1, img, !Sys.info()[1] %in% c("Linux","Windows"))) {
-    parallel <- 1
-    message("creating images is not compatible with parallel computing (parallel > 1) in OSX (mac)")
-  }
-  
-  if(any(parallel == 1, Sys.info()[1] == "Linux") & pb) {if(img) message("Creating spectrograms overlaid with dominant frequency measurements:") else
-    message("measuring dominant frequency:")}  
+  if(pb) if(img) cat("Creating spectrograms overlaid with dominant frequency measurements:") else
+    cat("measuring dominant frequency:") 
   
   dftsFUN <- function(X, i, bp, wl, threshold.time, threshold.freq, fsmooth, wl.freq, frange.detec){
     
     # Read sound files to get sample rate and length
     r <- tuneR::readWave(as.character(X$sound.files[i]), header = TRUE)
     f <- r$sample.rate
-
+    
     #in case bp its higher than can be due to sampling rate
     b <- bp 
     if(!is.null(b)) {if(b[2] > ceiling(f/2000) - 1) b[2] <- ceiling(f/2000) - 1 
     b <- b * 1000}
     
-      r <- tuneR::readWave(as.character(X$sound.files[i]), from = X$start[i], to = X$end[i], units = "seconds")
+    r <- tuneR::readWave(as.character(X$sound.files[i]), from = X$start[i], to = X$end[i], units = "seconds")
     
-      if(frange.detec)
-      {frng <- frd_wrblr_int(wave = r, wl = wl.freq, fsmooth = fsmooth, threshold = threshold.freq, wn = wn, flim = c(0, 22), bp = b/ 1000, ovlp = ovlp)
-      
-      if(!all(is.na(frng$frange))) b <- as.numeric(frng$frange) * 1000 }
-      
-      # calculate dominant frequency at each time point     
-      dfreq1 <- seewave::dfreq(r, f = f, wl = wl, plot = FALSE, ovlp = ovlp, bandpass = b, fftw = TRUE, 
-                              threshold = threshold)
-      
-      dfreq <- dfreq1[!is.na(dfreq1[,2]), ]
-      dfreq <- dfreq[dfreq[,2] > b[1]/1000, ]
-      
-      if(nrow(dfreq) < 2) {apdom <- list()
-      apdom$x <- dfreq1[, 1]
-      apdom$y <- rep(NA, length.out)
+    if(frange.detec)
+    {frng <- frd_wrblr_int(wave = r, wl = wl.freq, fsmooth = fsmooth, threshold = threshold.freq, wn = wn, flim = c(0, 22), bp = b/ 1000, ovlp = ovlp)
+    
+    if(!all(is.na(frng$frange))) b <- as.numeric(frng$frange) * 1000 }
+    
+    # calculate dominant frequency at each time point     
+    dfreq1 <- seewave::dfreq(r, f = f, wl = wl, plot = FALSE, ovlp = ovlp, bandpass = b, fftw = TRUE, 
+                             threshold = threshold)
+    
+    dfreq <- dfreq1[!is.na(dfreq1[,2]), ]
+    dfreq <- dfreq[dfreq[,2] > b[1]/1000, ]
+    
+    if(nrow(dfreq) < 2) {apdom <- list()
+    apdom$x <- dfreq1[, 1]
+    apdom$y <- rep(NA, length.out)
+    apdom1 <- apdom
+    
+    } else {
+      if(!clip.edges)
+      {        apdom <- approx(dfreq[,1], dfreq[,2], xout = seq(from = dfreq1[1, 1], 
+                                                                to = dfreq1[nrow(dfreq1), 1], length.out = length.out),
+                               method = "linear")
       apdom1 <- apdom
-      
-       } else {
-        if(!clip.edges)
-{        apdom <- approx(dfreq[,1], dfreq[,2], xout = seq(from = dfreq1[1, 1], 
-            to = dfreq1[nrow(dfreq1), 1], length.out = length.out),
-            method = "linear")
-      apdom1 <- apdom
-          } else {
-              apdom <- approx(dfreq[,1], dfreq[,2], 
-              xout = seq(from = dfreq[1, 1],  to = dfreq[nrow(dfreq), 1], 
-              length.out = length.out), method = "linear")
-            
-      #fix for ploting with trackfreqs
-      dfreq1[,2][is.na(dfreq1[,2])] <- 0
-      
-      #calculate time at start and end with no amplitude detected (duration of clipped edges)
-      durend1 <- suppressWarnings(diff(range(dfreq1[,1][rev(cumsum(rev(dfreq1[,2])) == 0)])))
-      durend <- durend1
-      if(is.infinite(durend) | is.na(durend)) durend <- 0
-      
-      durst1 <- suppressWarnings(diff(range(dfreq1[,1][cumsum(dfreq1[,2]) == 0])))   
-      durst <- durst1
-      if(is.infinite(durst) | is.na(durst)) durst <- 0
-      
-      by.dur <- mean(diff(apdom$x))
-      clipst <- length(seq(from = 0, to = durst, by = by.dur))
-      clipend <- length(seq(from = 0, to = durend, by = by.dur))
-      
-      apdom1 <- apdom
-      apdom1$y <- c(rep(NA, clipst) ,apdom$y, rep(NA, clipend))
-      
-      if(is.infinite(durst1) | is.na(durst1)) apdom1$y <- apdom1$y[-1]
-      if(is.infinite(durend1) | is.na(durend1)) apdom1$y <- apdom1$y[-length(apdom1$y)]
-               } 
-          }
-      
-      
-  if(img) 
+      } else {
+        apdom <- approx(dfreq[,1], dfreq[,2], 
+                        xout = seq(from = dfreq[1, 1],  to = dfreq[nrow(dfreq), 1], 
+                                   length.out = length.out), method = "linear")
+        
+        #fix for ploting with trackfreqs
+        dfreq1[,2][is.na(dfreq1[,2])] <- 0
+        
+        #calculate time at start and end with no amplitude detected (duration of clipped edges)
+        durend1 <- suppressWarnings(diff(range(dfreq1[,1][rev(cumsum(rev(dfreq1[,2])) == 0)])))
+        durend <- durend1
+        if(is.infinite(durend) | is.na(durend)) durend <- 0
+        
+        durst1 <- suppressWarnings(diff(range(dfreq1[,1][cumsum(dfreq1[,2]) == 0])))   
+        durst <- durst1
+        if(is.infinite(durst) | is.na(durst)) durst <- 0
+        
+        by.dur <- mean(diff(apdom$x))
+        clipst <- length(seq(from = 0, to = durst, by = by.dur))
+        clipend <- length(seq(from = 0, to = durend, by = by.dur))
+        
+        apdom1 <- apdom
+        apdom1$y <- c(rep(NA, clipst) ,apdom$y, rep(NA, clipend))
+        
+        if(is.infinite(durst1) | is.na(durst1)) apdom1$y <- apdom1$y[-1]
+        if(is.infinite(durend1) | is.na(durend1)) apdom1$y <- apdom1$y[-length(apdom1$y)]
+      } 
+    }
+    
+    
+    if(img) 
       trackfreqs(X[i,], wl = wl, wl.freq = wl.freq, osci = FALSE, leglab = leglab, pb = FALSE, wn = wn, threshold.time = threshold.time, threshold.freq = threshold.freq, bp = bp, 
                  parallel = 1, path = path, img.suffix = img.suffix, ovlp = ovlp,
                  custom.contour = data.frame(sound.files = X$sound.files[i], selec = X$selec[i], t(apdom1$y)), xl = ifelse(frange.detec, 1.8, 1), fsmooth = fsmooth, frange.detec = frange.detec, ...)
@@ -241,57 +224,22 @@ dfts <-  function(X, wl = 512, wl.freq = 512, length.out = 20, wn = "hanning", o
     return(apdom$y)  
   } 
 
-  # Run parallel in windows
-  if(parallel > 1) {
-    if(Sys.info()[1] == "Windows") {
-      
-      i <- NULL #only to avoid non-declared objects
-      
-      cl <- parallel::makeCluster(parallel)
-      
-      doParallel::registerDoParallel(cl)
-      
-      lst <- foreach::foreach(i = 1:nrow(X)) %dopar% {
-        dftsFUN(X, i, bp, wl, threshold.time, threshold.freq, fsmooth, wl.freq, frange.detec)
-      }
-      
-      parallel::stopCluster(cl)
-      
-    } 
-    if(Sys.info()[1] == "Linux") {    # Run parallel in Linux
-      
-     if(pb)
-       lst <- pbmcapply::pbmclapply(1:nrow(X), mc.cores = parallel, function (i) {
-         dftsFUN(X, i, bp, wl, threshold.time, threshold.freq, fsmooth, wl.freq, frange.detec)
-       }) else
-       lst <- parallel::mclapply(1:nrow(X), mc.cores = parallel, function (i) {
-        dftsFUN(X, i, bp, wl, threshold.time, threshold.freq, fsmooth, wl.freq, frange.detec)
-      })
-    }
-    if(!any(Sys.info()[1] == c("Linux", "Windows"))) # parallel in OSX
-    {
-      cl <- parallel::makeForkCluster(getOption("cl.cores", parallel))
-      
-      doParallel::registerDoParallel(cl)
-      
-      lst <- foreach::foreach(i = 1:nrow(X)) %dopar% {
-        dftsFUN(X, i, bp, wl, threshold.time, threshold.freq, fsmooth, wl.freq, frange.detec)
-      }
-      
-      parallel::stopCluster(cl)
-      
-    }
-  } else {
-   if(pb)
-     lst <- pbapply::pblapply(1:nrow(X), function(i) dftsFUN(X, i, bp, wl, threshold.time, threshold.freq, fsmooth, wl.freq, frange.detec)) else
-       lst <- lapply(1:nrow(X), function(i) dftsFUN(X, i, bp, wl, threshold.time, threshold.freq, fsmooth, wl.freq, frange.detec))
-  }
+  # set pb options 
+  pbapply::pboptions(type = ifelse(pb, "timer", "none"))
   
+  # set clusters for windows OS
+  if (Sys.info()[1] == "Windows" & parallel > 1)
+    cl <- parallel::makePSOCKcluster(getOption("cl.cores", parallel)) else cl <- parallel
+  
+  # run loop apply function
+  lst <- pbapply::pblapply(X = 1:nrow(X), cl = cl, FUN = function(i) 
+  { 
+ dftsFUN(X, i, bp, wl, threshold.time, threshold.freq, fsmooth, wl.freq, frange.detec)
+  }) 
   
   df <- data.frame(sound.files = X$sound.files, selec = X$selec, (as.data.frame(matrix(unlist(lst),nrow = length(X$sound.files), byrow = TRUE))))
     colnames(df)[3:ncol(df)]<-paste("dfreq",1:(ncol(df)-2),sep = "-")
             
     df[ ,3:ncol(df)] <- round(df[ ,3:ncol(df)], 3)
          return(df)
-
     }
