@@ -24,12 +24,10 @@
 #' @param img Logical argument. If \code{FALSE}, image files are not produced. Default is \code{TRUE}.
 #' @param parallel Numeric. Controls whether parallel computing is applied.
 #'  It specifies the number of cores to be used. Default is 1 (i.e. no parallel computing).
-#'  Not available in Windows OS.
 #' @param path Character string containing the directory path where the sound files are located.
 #' @param img.suffix A character vector of length 1 with a sufix (label) to add at the end of the names of 
 #' image files.
-#' @param pb Logical argument to control progress bar. Default is \code{TRUE}. Note that progress bar is only used
-#' when parallel = 1.
+#' @param pb Logical argument to control progress bar. Default is \code{TRUE}.
 #' @param clip.edges Logical argument to control whether edges (start or end of signal) in
 #' which amplitude values above the threshold were not detected will be removed. If 
 #' \code{TRUE} this edges will be excluded and signal contour will be calculated on the
@@ -81,7 +79,7 @@
 #' 
 #' }
 #' @author Marcelo Araya-Salas (\email{araya-salas@@cornell.edu})
-#last modification on oct-26-2016 (MAS)
+#last modification on mar-13-2018 (MAS)
 
 sp.en.ts <-  function(X, wl = 512, length.out = 20, wn = "hanning", ovlp = 70, 
                   bp = NULL, threshold = 15, img = TRUE, parallel = 1,
@@ -100,8 +98,6 @@ sp.en.ts <-  function(X, wl = 512, length.out = 20, wn = "hanning", ovlp = 70,
   #if X is not a data frame
   if(!class(X) %in% c("data.frame", "selection.table")) stop("X is not of a class 'data.frame' or 'selection table")
   
-  
-  
   if(!all(c("sound.files", "selec", 
             "start", "end") %in% colnames(X))) 
     stop(paste(paste(c("sound.files", "selec", "start", "end")[!(c("sound.files", "selec", 
@@ -136,7 +132,7 @@ sp.en.ts <-  function(X, wl = 512, length.out = 20, wn = "hanning", ovlp = 70,
   #return warning if not all sound files were found
   recs.wd <- list.files(pattern = "\\.wav$", ignore.case = TRUE)
   if(length(unique(X$sound.files[(X$sound.files %in% recs.wd)])) != length(unique(X$sound.files)) & pb) 
-    message(paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% recs.wd)])), 
+    cat(paste(length(unique(X$sound.files))-length(unique(X$sound.files[(X$sound.files %in% recs.wd)])), 
                   ".wav file(s) not found"))
   
   #count number of sound files in working directory and if 0 stop
@@ -150,19 +146,8 @@ sp.en.ts <-  function(X, wl = 512, length.out = 20, wn = "hanning", ovlp = 70,
   if(!is.numeric(parallel)) stop("'parallel' must be a numeric vector of length 1") 
   if(any(!(parallel %% 1 == 0),parallel < 1)) stop("'parallel' should be a positive integer")
   
-  #if parallel
-  if(all(parallel > 1, img, !Sys.info()[1] %in% c("Linux","Windows"))) {
-    parallel <- 1
-    message("creating images is not compatible with parallel computing (parallel > 1) in OSX (mac)")
-  }
-  
-  #parallel not available on windows
-  if(parallel > 1 & Sys.info()[1] == "Windows")
-  {message("parallel computing not availabe in Windows OS for this function")
-    parallel <- 1}
-
- if(pb) {if(img) message("Creating spectrograms overlaid with dominant frequency measurements:") else
-    message("Measuring spectral entropy:")}  
+ if(pb) {if(img) cat("Creating spectrograms overlaid with dominant frequency measurements:") else
+    cat("Measuring spectral entropy:")}  
   
   sp.en.tsFUN <- function(X, i, bp, wl, threshold, sp.en.range){
     
@@ -220,55 +205,24 @@ sp.en.ts <-  function(X, wl = 512, length.out = 20, wn = "hanning", ovlp = 70,
     return(apen$y)  
   } 
 
-  # Run parallel in windows
-  if(parallel > 1) {
-    if(Sys.info()[1] == "Windows") {
-      
-      i <- NULL #only to avoid non-declared objects
-      
-      cl <- parallel::makeCluster(parallel)
-      
-      doParallel::registerDoParallel(cl)
-      
-      lst <- foreach::foreach(i = 1:nrow(X)) %dopar% {
-        sp.en.tsFUN(X, i, bp, wl, threshold, sp.en.range)
-      }
-      
-      parallel::stopCluster(cl)
-      
-    } 
-    if(Sys.info()[1] == "Linux") {    # Run parallel in Linux
-      if(pb)
-        lst <- pbmcapply::pbmclapply(1:nrow(X), mc.cores = parallel, function (i) {
-          sp.en.tsFUN(X, i, bp, wl, threshold, sp.en.range)
-        }) else
-      lst <- parallel::mclapply(1:nrow(X), mc.cores = parallel, function (i) {
-        sp.en.tsFUN(X, i, bp, wl, threshold, sp.en.range)
-      })
-    }
-    if(!any(Sys.info()[1] == c("Linux", "Windows"))) # parallel in OSX
-    {
-      cl <- parallel::makeForkCluster(getOption("cl.cores", parallel))
-      
-      doParallel::registerDoParallel(cl)
-      
-      lst <- foreach::foreach(i = 1:nrow(X)) %dopar% {
-        sp.en.tsFUN(X, i, bp, wl, threshold, sp.en.range)
-      }
-      
-      parallel::stopCluster(cl)
-      
-    }
-  } else {
-   if(pb)
-     lst <- pbapply::pblapply(1:nrow(X), function(i) sp.en.tsFUN(X, i, bp, wl, threshold, sp.en.range)) else
-       lst <- lapply(1:nrow(X), function(i) sp.en.tsFUN(X, i, bp, wl, threshold, sp.en.range))
-  }
+  # set pb options 
+  pbapply::pboptions(type = ifelse(pb, "timer", "none"))
   
+  # set clusters for windows OS
+  if (Sys.info()[1] == "Windows" & parallel > 1)
+    cl <- parallel::makePSOCKcluster(getOption("cl.cores", parallel)) else cl <- parallel
   
+  # run loop apply function
+  lst <- pbapply::pblapply(X = 1:nrow(X), cl = cl, FUN = function(i) 
+  { 
+    sp.en.tsFUN(X, i, bp, wl, threshold, sp.en.range)
+  }) 
+  
+  # put data together
   df <- data.frame(sound.files = X$sound.files, selec = X$selec, (as.data.frame(matrix(unlist(lst),nrow = length(X$sound.files), byrow = TRUE))))
-    colnames(df)[3:ncol(df)]<-paste("sp.en",1:(ncol(df)-2),sep = "-")
+    colnames(df)[3:ncol(df)] <- paste("sp.en",1:(ncol(df)-2),sep = "-")
     df[ ,3:ncol(df)] <- round(df[ ,3:ncol(df)], 3)   
-              return(df)
 
-    }
+  return(df)
+
+}
